﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EnvelopeCreatorProject.DocusignAPI;
using EnvelopeCreatorProject.DocusignAPI.RecipientType;
using EnvelopeCreatorProject.Models.Recipients;
using EnvelopeCreatorProject.Models.Recipients.RecipientTypeBuilders;
using EnvelopeCreatorProject.Models.Signers;
namespace EnvelopeCreatorProject.Models.Builders
{
    internal class EnvelopeRecipientBuilder : IEnvelopeBuilder
    {
        private readonly List<Recipient> _recipients;
        private readonly List<Tab> _tabs;
        private readonly List<Tab> _dateSignedFields;
        private readonly HostSigner _host;

        public EnvelopeRecipientBuilder(List<Recipient> recipients, List<Tab> tabs, List<Tab> dateSignedFields, HostSigner host)
        {
            _recipients = recipients;
            _tabs = tabs;
            _dateSignedFields = dateSignedFields;
            _host = host;
        }

        public void BuildTo(StringBuilder strBuilder)
        {
            var signTabs = _tabs.Where(t => t.Type == TabTypeCode.SignHere).ToList();
            var initialsTabs = _tabs.Where(t => t.Type == TabTypeCode.InitialHere).ToList();

            var signHandlers = new Dictionary<RecipientTypeCode, RecipientTypeBuilder>()
            {
                {RecipientTypeCode.CarbonCopy, new CarbonCopiesBuilder()},
                {RecipientTypeCode.Signer, (new SignHereSignerBuilder(signTabs, initialsTabs, _dateSignedFields))},
                {RecipientTypeCode.InPersonSigner, (new InPersonSignersBuilder(signTabs, initialsTabs, _dateSignedFields, _host))},
                {RecipientTypeCode.Agent, new AgentsBuilder()},
                {RecipientTypeCode.CertifiedDelivery,new CertifiedDeliveriesBuilder()},
                {RecipientTypeCode.Editor, new EditorsBuilder()},
                {RecipientTypeCode.Intermediary, new IntermediariesBuilder()}
            };


            _recipients.ForEach(r => signHandlers[r.GetType()].AddRecipientData(r));
            
            strBuilder.AppendLine("\"recipients\":{");
            strBuilder.AppendLine(String.Join(",", _recipients.Select(r => signHandlers[r.GetType()].GetRecipientsData())));
            strBuilder.AppendLine("}");
        }
    }
}
