﻿using System;
using System.Collections.Generic;
using System.Linq;
using EnvelopeCreatorProject.DocusignAPI;


namespace EnvelopeCreatorProject.Models.Recipients
{
    internal class SignerRecipientsBuilder:RecipientTypeBuilder
    {
        private readonly List<Tab> _signHereTabs;
        private readonly List<Tab> _initialsTabs;
        private readonly List<Tab> _dataSignFields;

        internal SignerRecipientsBuilder(List<Tab> signHereTabs, List<Tab> initialsTabs, List<Tab> dataSignFields)
        {
            _signHereTabs = signHereTabs;
            _initialsTabs = initialsTabs;
            _dataSignFields = dataSignFields;        
        }

        protected override string GetData(Recipient objRecipient)
        {
            return string.Empty;
        }

        protected string GetTabs(List<Tab> tabs, Recipient objRecipient, string name)
        {
            var recipientTabs = tabs.FindAll((Tab tab) => tab.RecipientID == objRecipient.ID);
            return !recipientTabs.Any() ? String.Empty : string.Format("{0}{1}{0}:[{2}]", "\"", name, String.Join(",", recipientTabs.Select(getTabMetaData)));
        }

        protected string getTabMetaData(Tab tab)
        {
            return string.Format(@"{{{0}xPosition{0}:{0}{1}{0},
                                    {0}yPosition{0}:{0}{2}{0}, 
                                 {0}documentId{0}:{0}{3}{0},
                                {0}pageNumber{0}:{0}{4}{0},
                                {0}recipientId{0}:{0}{5}{0},
                                {0}optional{0}:{6}}}",
                "\"",
                tab.XPosition,
                tab.YPosition,
                tab.DocumentID,
                tab.PageNumber,
                tab.RecipientID,
                tab.Optional.ToString().ToLower());
        }

        protected string GetSignatureInfoTo(Recipient objRecipient)
        {
            if (objRecipient.SignatureInfo == null)
                return string.Empty;

            return string.Format(
                 "{0}signatureInfo{0}:{{{0}fontStyle{0}:{0}{1}{0},{0}signatureName{0}:{0}{2}{0},{0}signatureInitials{0}:{0}{3}{0}}}",
                 "\"",
                 objRecipient.SignatureInfo.FontStyle.ToString(),
                 objRecipient.SignatureInfo.SignatureName,
                 objRecipient.SignatureInfo.SignatureInitials
                 );
        }

        protected string GetSignTabs(Recipient objRecipient)
        {
            return string.Format("{0}tabs{0}:{{{1}}}",
                "\"",
                GetTabSection(
                    GetTabs(_signHereTabs, objRecipient, "signHereTabs"), //First, add SignHere Tabs (FullName as signature)
                    GetTabs(_initialsTabs, objRecipient, "initialHereTabs"), //Now add SignInitials Tabs
                    GetTabs(_dataSignFields, objRecipient, "dateSignedTabs") //Finally, add Date Signed Tabs
                )  
            );
        }

        private string GetTabSection(params string[] tabs)
        {
            return string.Join(",", tabs.AsEnumerable().Where(t => !string.IsNullOrEmpty(t)));
        }
    }
}
