﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Net;
using System.Text;
using EnvelopeCreatorProject;
using EnvelopeCreatorProject.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using QuikESignTransport.SelfService.ContractResolvers;
using QuikESignTransport.SelfService.Recipients;

namespace QuikESignTransport.SelfService.Classes
{
    internal class RequestHelper
    {
        internal SelfServiceData GetEnvelopeData(string jsonString, string quikToken)
        {
            var headers = new List<KeyValuePair<string, string>>()
            {
                new KeyValuePair<string, string>("Authorization", string.Format("BEARER {0}", quikToken))
            };

            var result = DoRequest(GetQuikDocusignSelfServiceEndpoint(), jsonString, headers);
            
            var jsonResponse = JObject.Parse(result);
            
            var deserializerSettings = new JsonSerializerSettings()
            {
                ContractResolver = new PrivateMemberContractResolver()
            };

            if (jsonResponse["ErrorCode"].ToObject<int>() != 0)
                throw new Exception("Error getting self service data from Esignature endpoint. See the details in the response: " + result);
            
            var selfServiceData = JsonConvert.DeserializeObject<SelfServiceData>(jsonResponse["ResultData"].ToString(), deserializerSettings);

            //Build the Recipient Type from the SendType
            selfServiceData.EnvelopeData.Recipients.ForEach(r => RecipientBuilder.GetSendTypeBuilder(SendType.GetSendType(r.SendType)).Build(r));

            return selfServiceData;
        }

        internal string GenerateQuikToken()
        {
            var body = String.Format("grant_type=password&username={0}&password={1}", GetUsername(), GetPassword());
            var response = DoRequest(GetQuikOAuthTokenEndpoint(), body, null, "application/x-www-form-urlencoded");
            return JObject.Parse(response)["access_token"].ToString();
        }

        internal AccountAuthentication GetDocusignAuthentication()
        {
            return new AccountAuthentication()
            {
                AuthToken = GetAuthToken(),
                SenderToken = "",
                SOBOFlag = false,
                EnvironmentURL = GetEnvirnmentURl()
            };
        }

        internal EnvelopeResult GenerateDocusignEnvelope(SelfServiceData selfServiceData, string integratorKey, string APIVersion)
        {
            var envelopeCreator = new EnvelopeTransporterCreator(integratorKey, APIVersion);
            return envelopeCreator.CreateEnvelope(selfServiceData.AccountAuthentication, selfServiceData.EnvelopeData);
        }

        private string GetUsername()
        {
            return ConfigurationManager.AppSettings["QUIK_USERNAME"];
        }

        private string GetPassword()
        {
            return ConfigurationManager.AppSettings["QUIK_PASSWORD"];
        }

        private string GetQuikOAuthTokenEndpoint()
        {
            return ConfigurationManager.AppSettings["QUIK_OAUTHTOKEN_ENDPOINT"];
        }

        private string GetQuikDocusignSelfServiceEndpoint()
        {
            return ConfigurationManager.AppSettings["QUIK_DOCUSIGN_SELFSERVICE_ENDPOINT"];
        }

        private string GetAuthToken()
        {
            return ConfigurationManager.AppSettings["DOCUSIGN_AUTHTOKEN"];
        }
        private string GetEnvirnmentURl()
        {
            return ConfigurationManager.AppSettings["DOCUSIGN_ENVIRONMENT_URL"];
        }
        
        private string DoRequest(string url, string body, List<KeyValuePair<string, string>> headers, string contentType = "application/json")
        {
            var request = HttpWebRequest.Create(url);
            request.ContentType = contentType;
            request.Method = "POST";

            if (headers != null)
            {
                foreach (var header in headers)
                {
                    request.Headers.Add(header.Key, header.Value);
                }
            }

            var streamBody = Encoding.UTF8.GetBytes(body);
            request.ContentLength = streamBody.Length;
            using (var requestDataStream = request.GetRequestStream())
            {
                requestDataStream.Write(streamBody, 0, streamBody.Length);
                using (var response = request.GetResponse())
                {
                    var responseDataStream = response.GetResponseStream();
                    var responseString = new StreamReader(responseDataStream).ReadToEnd();
                    return responseString;
                }
            }

        }
    }
}